@@ -31,20 +31,27 @@
 
 #ifdef HAVE_RADIUS
 
-#include "cfg.h"
-#include <freeradius-client.h>
+#include "common-config.h"
+
+#ifdef LEGACY_RADIUS
+# include <freeradius-client.h>
+#else
+# include <radcli/radcli.h>
+#endif
 
 #define RAD_GROUP_NAME 1030
 #define RAD_IPV4_DNS1 ((311<<16)|(28))
 #define RAD_IPV4_DNS2 ((311<<16)|(29))
 
-#ifndef PW_INTERIM_INTERVAL
-# define PW_INTERIM_INTERVAL 85
+#if defined(LEGACY_RADIUS)
+# ifndef PW_DELEGATED_IPV6_PREFIX
+#  define PW_DELEGATED_IPV6_PREFIX 123
+# endif
+# ifndef PW_ACCT_INTERIM_INTERVAL
+#  define PW_ACCT_INTERIM_INTERVAL 85
+# endif
 #endif
 
-#ifndef PW_DELEGATED_IPV6_PREFIX
-# define PW_DELEGATED_IPV6_PREFIX 123
-#endif
 
 static rc_handle *rh = NULL;
 static char nas_identifier[64];
@@ -195,14 +202,14 @@ static int radius_auth_pass(void *ctx, const char *pass, unsigned pass_len)
 	syslog(LOG_DEBUG, "radius-auth: communicating username (%s) and password", pctx->username);
 	if (rc_avpair_add(rh, &send, PW_USER_NAME, pctx->username, -1, 0) == NULL) {
 		syslog(LOG_ERR,
-		       "%s:%u: user '%s' auth error", __func__, __LINE__,
+		       "%s:%u: error in constructing radius message for user '%s'", __func__, __LINE__,
 		       pctx->username);
 		return ERR_AUTH_FAIL;
 	}
 
 	if (rc_avpair_add(rh, &send, PW_USER_PASSWORD, (char*)pass, -1, 0) == NULL) {
 		syslog(LOG_ERR,
-		       "%s:%u: user '%s' auth error", __func__, __LINE__,
+		       "%s:%u: error in constructing radius message for user '%s'", __func__, __LINE__,
 		       pctx->username);
 		ret = ERR_AUTH_FAIL;
 		goto cleanup;
@@ -223,7 +230,7 @@ static int radius_auth_pass(void *ctx, const char *pass, unsigned pass_len)
 	if (nas_identifier[0] != 0) {
 		if (rc_avpair_add(rh, &send, PW_NAS_IDENTIFIER, nas_identifier, -1, 0) == NULL) {
 			syslog(LOG_ERR,
-			       "%s:%u: user '%s' auth error", __func__, __LINE__,
+			       "%s:%u: error in constructing radius message for user '%s'", __func__, __LINE__,
 			       pctx->username);
 			ret = ERR_AUTH_FAIL;
 			goto cleanup;
@@ -232,7 +239,7 @@ static int radius_auth_pass(void *ctx, const char *pass, unsigned pass_len)
 
 	if (rc_avpair_add(rh, &send, PW_CALLING_STATION_ID, pctx->remote_ip, -1, 0) == NULL) {
 		syslog(LOG_ERR,
-		       "%s:%u: user '%s' auth error", __func__, __LINE__,
+		       "%s:%u: error in constructing radius message for user '%s'", __func__, __LINE__,
 		       pctx->username);
 		ret = ERR_AUTH_FAIL;
 		goto cleanup;
@@ -241,7 +248,7 @@ static int radius_auth_pass(void *ctx, const char *pass, unsigned pass_len)
 	service = PW_AUTHENTICATE_ONLY;
 	if (rc_avpair_add(rh, &send, PW_SERVICE_TYPE, &service, -1, 0) == NULL) {
 		syslog(LOG_ERR,
-		       "%s:%u: user '%s' auth error", __func__, __LINE__,
+		       "%s:%u: error in constructing radius message for user '%s'", __func__, __LINE__,
 		       pctx->username);
 		ret = ERR_AUTH_FAIL;
 		goto cleanup;
@@ -251,7 +258,7 @@ static int radius_auth_pass(void *ctx, const char *pass, unsigned pass_len)
 		service = pctx->id;
 		if (rc_avpair_add(rh, &send, PW_NAS_PORT, &service, -1, 0) == NULL) {
 			syslog(LOG_ERR,
-			       "%s:%u: user '%s' auth error", __func__, __LINE__,
+			       "%s:%u: error in constructing radius message for user '%s'", __func__, __LINE__,
 			       pctx->username);
 			ret = ERR_AUTH_FAIL;
 			goto cleanup;
@@ -261,7 +268,7 @@ static int radius_auth_pass(void *ctx, const char *pass, unsigned pass_len)
 	service = PW_ASYNC;
 	if (rc_avpair_add(rh, &send, PW_NAS_PORT_TYPE, &service, -1, 0) == NULL) {
 		syslog(LOG_ERR,
-		       "%s:%u: user '%s' auth error", __func__, __LINE__,
+		       "%s:%u: error in constructing radius message for user '%s'", __func__, __LINE__,
 		       pctx->username);
 		ret = ERR_AUTH_FAIL;
 		goto cleanup;
@@ -335,7 +342,7 @@ static int radius_auth_pass(void *ctx, const char *pass, unsigned pass_len)
 			} else if (vp->attribute == PW_FRAMED_IPV6_ROUTE && vp->type == PW_TYPE_STRING) {
 				/* Framed-IPv6-Route */
 				append_route(pctx, vp->strvalue, vp->lvalue);
-			} else if (vp->attribute == PW_INTERIM_INTERVAL && vp->type == PW_TYPE_INTEGER) {
+			} else if (vp->attribute == PW_ACCT_INTERIM_INTERVAL && vp->type == PW_TYPE_INTEGER) {
 				pctx->interim_interval_secs = vp->lvalue;
 			} else if (vp->attribute == PW_SESSION_TIMEOUT && vp->type == PW_TYPE_INTEGER) {
 				pctx->session_timeout_secs = vp->lvalue;
