@@ -40,6 +40,9 @@
 #include <sys/socket.h>
 #include <sys/un.h>
 
+static
+int common_info_cmd(UserListRep *args, FILE *out, cmd_params_st *params);
+
 struct unix_ctx {
 	int fd;
 	int is_open;
@@ -210,18 +213,23 @@ error:
 
 }
 
-int handle_status_cmd(struct unix_ctx *ctx, const char *arg)
+int handle_status_cmd(struct unix_ctx *ctx, const char *arg, cmd_params_st *params)
 {
 	int ret;
 	struct cmd_reply_st raw;
 	StatusRep *rep;
 	char str_since[64];
+	char buf[MAX_TMPSTR_SIZE];
 	time_t t;
 	struct tm *tm;
 	PROTOBUF_ALLOCATOR(pa, ctx);
 
 	init_reply(&raw);
 
+	print_start_block(stdout, params);
+	if (NO_JSON(params))
+		printf("OpenConnect SSL VPN server\n");
+
 	ret = send_cmd(ctx, CTL_CMD_STATUS, NULL, NULL, NULL, &raw);
 	if (ret < 0) {
 		goto error_status;
@@ -231,23 +239,23 @@ int handle_status_cmd(struct unix_ctx *ctx, const char *arg)
 	if (rep == NULL)
 		goto error_status;
 
-	printf("OpenConnect SSL VPN server\n");
-	printf("           Status: %s\n", rep->status != 0 ? "online" : "error");
+
+	print_single_value(stdout, params, "Status", rep->status != 0 ? "online" : "error", 1);
 
 	t = rep->start_time;
 	tm = localtime(&t);
+	print_time_ival7(buf, time(0), t);
 	strftime(str_since, sizeof(str_since), DATE_TIME_FMT, tm);
-	printf("         Up since: %s (", str_since);
-	print_time_ival7(time(0), t, stdout);
-	fputs(")\n", stdout);
 
-	printf("               Clients: %u\n", (unsigned)rep->active_clients);
-	printf("Sec-mod client entries: %u\n", (unsigned)rep->secmod_client_entries);
-	printf("       IPs in ban list: %u\n", (unsigned)rep->banned_ips);
-	printf("        TLS DB entries: %u\n", (unsigned)rep->stored_tls_sessions);
-	printf("\n");
-	printf("            Server PID: %u\n", (unsigned)rep->pid);
-	printf("           Sec-mod PID: %u\n", (unsigned)rep->sec_mod_pid);
+	print_single_value_ex(stdout, params, "Up since", str_since, buf, 1);
+	print_single_value_int(stdout, params, "Clients", rep->active_clients, 1);
+	print_single_value_int(stdout, params, "Sec-mod client entries", rep->secmod_client_entries, 1);
+	print_single_value_int(stdout, params, "IPs in ban list", rep->banned_ips, 1);
+	print_single_value_int(stdout, params, "TLS DB entries", rep->stored_tls_sessions, 1);
+	print_separator(stdout, params);
+	print_single_value_int(stdout, params, "Server PID", rep->pid, 1);
+	print_single_value_int(stdout, params, "Sec-mod PID", rep->sec_mod_pid, 1);
+	print_end_block(stdout, params, 0);
 
 	status_rep__free_unpacked(rep, &pa);
 
@@ -255,8 +263,8 @@ int handle_status_cmd(struct unix_ctx *ctx, const char *arg)
 	goto cleanup;
 
  error_status:
-	printf("OpenConnect SSL VPN server\n");
-	printf("                Status: offline\n");
+	print_single_value(stdout, params, "Status", "offline", 0);
+	print_end_block(stdout, params, 0);
 	ret = 1;
 
  cleanup:
@@ -264,7 +272,7 @@ int handle_status_cmd(struct unix_ctx *ctx, const char *arg)
 	return ret;
 }
 
-int handle_reload_cmd(struct unix_ctx *ctx, const char *arg)
+int handle_reload_cmd(struct unix_ctx *ctx, const char *arg, cmd_params_st *params)
 {
 	int ret;
 	struct cmd_reply_st raw;
@@ -304,7 +312,7 @@ int handle_reload_cmd(struct unix_ctx *ctx, const char *arg)
 	return ret;
 }
 
-int handle_stop_cmd(struct unix_ctx *ctx, const char *arg)
+int handle_stop_cmd(struct unix_ctx *ctx, const char *arg, cmd_params_st *params)
 {
 	int ret;
 	struct cmd_reply_st raw;
@@ -344,7 +352,7 @@ int handle_stop_cmd(struct unix_ctx *ctx, const char *arg)
 	return ret;
 }
 
-int handle_unban_ip_cmd(struct unix_ctx *ctx, const char *arg)
+int handle_unban_ip_cmd(struct unix_ctx *ctx, const char *arg, cmd_params_st *params)
 {
 	int ret;
 	struct cmd_reply_st raw;
@@ -410,7 +418,7 @@ int handle_unban_ip_cmd(struct unix_ctx *ctx, const char *arg)
 	return ret;
 }
 
-int handle_disconnect_user_cmd(struct unix_ctx *ctx, const char *arg)
+int handle_disconnect_user_cmd(struct unix_ctx *ctx, const char *arg, cmd_params_st *params)
 {
 	int ret;
 	struct cmd_reply_st raw;
@@ -460,7 +468,7 @@ int handle_disconnect_user_cmd(struct unix_ctx *ctx, const char *arg)
 	return ret;
 }
 
-int handle_disconnect_id_cmd(struct unix_ctx *ctx, const char *arg)
+int handle_disconnect_id_cmd(struct unix_ctx *ctx, const char *arg, cmd_params_st *params)
 {
 	int ret;
 	struct cmd_reply_st raw;
@@ -515,7 +523,7 @@ int handle_disconnect_id_cmd(struct unix_ctx *ctx, const char *arg)
 	return ret;
 }
 
-int handle_list_users_cmd(struct unix_ctx *ctx, const char *arg)
+int handle_list_users_cmd(struct unix_ctx *ctx, const char *arg, cmd_params_st *params)
 {
 	int ret;
 	struct cmd_reply_st raw;
@@ -523,6 +531,7 @@ int handle_list_users_cmd(struct unix_ctx *ctx, const char *arg)
 	unsigned i;
 	const char *vpn_ip, *groupname, *username;
 	const char *dtls_ciphersuite;
+	char tmpbuf[MAX_TMPSTR_SIZE];
 	FILE *out;
 	time_t t;
 	struct tm *tm;
@@ -544,7 +553,13 @@ int handle_list_users_cmd(struct unix_ctx *ctx, const char *arg)
 	if (rep == NULL)
 		goto error;
 
-	for (i=0;i<rep->n_user;i++) {
+	if (HAVE_JSON(params)) {
+		common_info_cmd(rep, out, params);
+	} else for (i=0;i<rep->n_user;i++) {
+		username = rep->user[i]->username;
+		if (username == NULL || username[0] == 0)
+			username = NO_USER;
+
 		if (rep->user[i]->local_ip != NULL && rep->user[i]->local_ip[0] != 0)
 			vpn_ip = rep->user[i]->local_ip;
 		else
@@ -565,23 +580,19 @@ int handle_list_users_cmd(struct unix_ctx *ctx, const char *arg)
 		if (groupname == NULL || groupname[0] == 0)
 			groupname = NO_GROUP;
 
-		username = rep->user[i]->username;
-		if (username == NULL || username[0] == 0)
-			username = NO_USER;
+		print_time_ival7(tmpbuf, time(0), t);
 
 		fprintf(out, "%8d %8s %8s %14s %14s %6s ",
 			(int)rep->user[i]->id, username, groupname, rep->user[i]->ip, vpn_ip, rep->user[i]->tun);
 
-		print_time_ival7(time(0), t, out);
-
 		dtls_ciphersuite = rep->user[i]->dtls_ciphersuite;
 		if (dtls_ciphersuite != NULL && dtls_ciphersuite[0] != 0) {
 			if (strlen(dtls_ciphersuite) > 16 && strncmp(dtls_ciphersuite, "(DTLS", 5) == 0 &&
 			    strncmp(&dtls_ciphersuite[8], ")-(RSA)-", 8) == 0)
 				dtls_ciphersuite += 16;
-			fprintf(out, " %14s %9s\n", dtls_ciphersuite, rep->user[i]->status);
+			fprintf(out, "%s %14s %9s\n", tmpbuf, dtls_ciphersuite, rep->user[i]->status);
 		} else {
-			fprintf(out, " %14s %9s\n", "(no dtls)", rep->user[i]->status);
+			fprintf(out, "%s %14s %9s\n", tmpbuf, "(no dtls)", rep->user[i]->status);
 		}
 
 		entries_add(ctx, username, strlen(username), rep->user[i]->id);
@@ -605,13 +616,14 @@ int handle_list_users_cmd(struct unix_ctx *ctx, const char *arg)
 }
 
 static
-int handle_list_banned_cmd(struct unix_ctx *ctx, const char *arg, unsigned points)
+int handle_list_banned_cmd(struct unix_ctx *ctx, const char *arg, cmd_params_st *params, unsigned points)
 {
 	int ret;
 	struct cmd_reply_st raw;
 	BanListRep *rep = NULL;
 	unsigned i;
 	char str_since[64];
+	char tmpbuf[MAX_TMPSTR_SIZE];
 	FILE *out;
 	struct tm *tm;
 	time_t t;
@@ -632,10 +644,13 @@ int handle_list_banned_cmd(struct unix_ctx *ctx, const char *arg, unsigned point
 	if (rep == NULL)
 		goto error;
 
+	print_array_block(out, params);
+
 	for (i=0;i<rep->n_info;i++) {
 		if (rep->info[i]->ip == NULL)
 			continue;
 
+
 		/* add header */
 		if (points == 0) {
 			if (rep->info[i]->has_expires) {
@@ -646,28 +661,45 @@ int handle_list_banned_cmd(struct unix_ctx *ctx, const char *arg, unsigned point
 				continue;
 			}
 
-			if (i == 0) {
+			if (i == 0 && NO_JSON(params)) {
 				fprintf(out, "%14s %14s %30s\n",
 					"IP", "score", "expires");
 			}
+			print_start_block(out, params);
 
-			fprintf(out, "%14s %14u %30s (",
-				rep->info[i]->ip, (unsigned)rep->info[i]->score, str_since);
-			print_time_ival7(t, time(0), out);
-			fputs(")\n", out);
+			print_time_ival7(tmpbuf, t, time(0));
+
+			if (HAVE_JSON(params)) {
+				print_single_value(out, params, "IP", rep->info[i]->ip, 1);
+				print_single_value_ex(out, params, "Since", str_since, tmpbuf, 1);
+				print_single_value_int(out, params, "Score", rep->info[i]->score, 0);
+			} else {
+				fprintf(out, "%14s %14u %30s (%s)\n",
+					rep->info[i]->ip, (unsigned)rep->info[i]->score, str_since, tmpbuf);
+			}
 		} else {
-			if (i == 0) {
+			if (i == 0 && NO_JSON(params)) {
 				fprintf(out, "%14s %14s\n",
 					"IP", "score");
 			}
+			print_start_block(out, params);
 
-			fprintf(out, "%14s %14u\n",
-				rep->info[i]->ip, (unsigned)rep->info[i]->score);
+			if (HAVE_JSON(params)) {
+				print_single_value(out, params, "IP", rep->info[i]->ip, 1);
+				print_single_value_int(out, params, "Score", rep->info[i]->score, 0);
+			} else {
+				fprintf(out, "%14s %14u\n",
+					rep->info[i]->ip, (unsigned)rep->info[i]->score);
+			}
 		}
 
+		print_end_block(out, params, i<(rep->n_info-1)?1:0);
+
 		ip_entries_add(ctx, rep->info[i]->ip, strlen(rep->info[i]->ip));
 	}
 
+	print_end_array_block(out, params);
+
 	ret = 0;
 	goto cleanup;
 
@@ -685,52 +717,53 @@ int handle_list_banned_cmd(struct unix_ctx *ctx, const char *arg, unsigned point
 	return ret;
 }
 
-int handle_list_banned_ips_cmd(struct unix_ctx *ctx, const char *arg)
+int handle_list_banned_ips_cmd(struct unix_ctx *ctx, const char *arg, cmd_params_st *params)
 {
-	return handle_list_banned_cmd(ctx, arg, 0);
+	return handle_list_banned_cmd(ctx, arg, params, 0);
 }
 
-int handle_list_banned_points_cmd(struct unix_ctx *ctx, const char *arg)
+int handle_list_banned_points_cmd(struct unix_ctx *ctx, const char *arg, cmd_params_st *params)
 {
-	return handle_list_banned_cmd(ctx, arg, 1);
+	return handle_list_banned_cmd(ctx, arg, params, 1);
 }
 
-int print_list_entries(FILE* out, const char* name, char **val, unsigned vsize)
+
+static char *int2str(char tmpbuf[MAX_TMPSTR_SIZE], int i)
 {
-	const char * tmp;
-	unsigned int i = 0;
-
-	for (i=0;i<vsize;i++) {
-		tmp = val[i];
-		if (tmp != NULL) {
-			if (i==0)
-				fprintf(out, "%s %s\n", name, tmp);
-			else
-				fprintf(out, "\t\t%s\n", tmp);
-		}
-	}
-
-	return i;
+	tmpbuf[0] = 0;
+	snprintf(tmpbuf, MAX_TMPSTR_SIZE, "%d", i);
+	return tmpbuf;
 }
 
-int common_info_cmd(UserListRep * args)
+static
+int common_info_cmd(UserListRep * args, FILE *out, cmd_params_st *params)
 {
 	char *username = "";
 	char *groupname = "";
 	char str_since[64];
+	char tmpbuf[MAX_TMPSTR_SIZE];
 	struct tm *tm;
 	time_t t;
-	FILE *out;
 	unsigned at_least_one = 0;
 	int ret = 1, r;
 	unsigned i;
+	unsigned init_pager = 0;
 
-	out = pager_start();
+	if (out == NULL) {
+		out = pager_start();
+		init_pager = 1;
+	}
+
+	if (HAVE_JSON(params))
+		fprintf(out, "[\n");
 
 	for (i=0;i<args->n_user;i++) {
 		if (at_least_one > 0)
 			fprintf(out, "\n");
-		fprintf(out, "ID: %d\n", (int)args->user[i]->id);
+
+		print_start_block(out, params);
+
+		print_single_value_int(out, params, "ID", args->user[i]->id, 1);
 
 		t = args->user[i]->conn_time;
 		tm = localtime(&t);
@@ -740,96 +773,88 @@ int common_info_cmd(UserListRep * args)
 		if (username == NULL || username[0] == 0)
 			username = NO_USER;
 
-		fprintf(out, "\tUsername: %s  ", username);
 
 		groupname = args->user[i]->groupname;
 		if (groupname == NULL || groupname[0] == 0)
 			groupname = NO_GROUP;
 
-		fprintf(out, "Groupname: %s\n", groupname);
-		fprintf(out, "\tState: %s  ", args->user[i]->status);
-		fprintf(out, "Remote IP: %s\n", args->user[i]->ip);
+		print_pair_value(out, params, "Username", username, "Groupname", groupname, 1);
+		print_pair_value(out, params, "State", args->user[i]->status, "Remote IP", args->user[i]->ip, 1);
 
 		if (args->user[i]->local_ip != NULL && args->user[i]->local_ip[0] != 0 &&
 		    args->user[i]->remote_ip != NULL && args->user[i]->remote_ip[0] != 0) {
-			fprintf(out, "\tIPv4: %s  ", args->user[i]->local_ip);
-			fprintf(out, "P-t-P IPv4: %s\n", args->user[i]->remote_ip);
+			print_pair_value(out, params, "IPv4", args->user[i]->local_ip, "P-t-P IPv4", args->user[i]->remote_ip, 1);
 		}
 		if (args->user[i]->local_ip6 != NULL && args->user[i]->local_ip6[0] != 0 &&
 		    args->user[i]->remote_ip6 != NULL && args->user[i]->remote_ip6[0] != 0) {
-			fprintf(out, "\tIPv6: %s  ", args->user[i]->local_ip6);
-			fprintf(out, "P-t-P IPv6: %s\n", args->user[i]->remote_ip6);
+			print_pair_value(out, params, "IPv6", args->user[i]->local_ip6, "P-t-P IPv6", args->user[i]->remote_ip6, 1);
 		}
-		fprintf(out, "\tDevice: %s  ", args->user[i]->tun);
 
 		if (args->user[i]->has_mtu != 0)
-			fprintf(out, "MTU: %d\n", args->user[i]->mtu);
+			print_pair_value(out, params, "Device", args->user[i]->tun, "MTU", int2str(tmpbuf, args->user[i]->mtu), 1);
 		else
-			fprintf(out, "\n");
+			print_single_value(out, params, "Device", args->user[i]->tun, 1);
 
-		if (args->user[i]->user_agent != NULL && args->user[i]->user_agent[0] != 0)
-			fprintf(out, "\tUser-Agent: %s\n", args->user[i]->user_agent);
+		print_single_value(out, params, "User-Agent", args->user[i]->user_agent, 1);
 
 		if (args->user[i]->rx_per_sec > 0 || args->user[i]->tx_per_sec > 0) {
 			/* print limits */
-			char buf[32];
+			char buf1[32];
+			char buf2[32];
 
 			if (args->user[i]->rx_per_sec > 0 && args->user[i]->tx_per_sec > 0) {
-				bytes2human(args->user[i]->rx_per_sec, buf, sizeof(buf), NULL);
-				fprintf(out, "\tLimit RX: %s/sec  ", buf);
+				bytes2human(args->user[i]->rx_per_sec, buf1, sizeof(buf1), "/sec");
+				bytes2human(args->user[i]->tx_per_sec, buf2, sizeof(buf2), "/sec");
 
-				bytes2human(args->user[i]->tx_per_sec, buf, sizeof(buf), NULL);
-				fprintf(out, "TX: %s/sec\n", buf);
+				print_pair_value(out, params, "Limit RX", buf1, "TX", buf2, 1);
 			} else if (args->user[i]->tx_per_sec > 0) {
-				bytes2human(args->user[i]->tx_per_sec, buf, sizeof(buf), NULL);
-				fprintf(out, "\tLimit TX: %s/sec\n", buf);
+				bytes2human(args->user[i]->tx_per_sec, buf1, sizeof(buf1), "/sec");
+				print_single_value(out, params, "Limit TX", buf1, 1);
 			} else if (args->user[i]->rx_per_sec > 0) {
-				bytes2human(args->user[i]->rx_per_sec, buf, sizeof(buf), NULL);
-				fprintf(out, "\tLimit RX: %s/sec\n", buf);
+				bytes2human(args->user[i]->rx_per_sec, buf1, sizeof(buf1), "/sec");
+				print_single_value(out, params, "Limit RX", buf1, 1);
 			}
 		}
 
-		print_iface_stats(args->user[i]->tun, args->user[i]->conn_time, out);
+		print_iface_stats(args->user[i]->tun, args->user[i]->conn_time, out, params, 1);
 
-		if (args->user[i]->hostname != NULL && args->user[i]->hostname[0] != 0)
-			fprintf(out, "\tHostname: %s\n", args->user[i]->hostname);
+		print_single_value(out, params, "Hostname", args->user[i]->hostname, 1);
 
-		fprintf(out, "\tConnected at: %s (", str_since);
-		print_time_ival7(time(0), t, out);
-		fprintf(out, ")\n");
+		print_time_ival7(tmpbuf, time(0), t);
+		print_single_value_ex(out, params, "Connected at", str_since, tmpbuf, 1);
 
-		fprintf(out, "\tTLS ciphersuite: %s\n", args->user[i]->tls_ciphersuite);
-		if (args->user[i]->dtls_ciphersuite != NULL && args->user[i]->dtls_ciphersuite[0] != 0)
-			fprintf(out, "\tDTLS cipher: %s\n", args->user[i]->dtls_ciphersuite);
-
-		if (args->user[i]->cstp_compr && args->user[i]->cstp_compr[0] != 0)
-			fprintf(out, "\tCSTP compression: %s\n", args->user[i]->cstp_compr);
-		if (args->user[i]->dtls_compr != NULL && args->user[i]->dtls_compr[0] != 0)
-			fprintf(out, "\tDTLS compression: %s\n", args->user[i]->dtls_compr);
+		print_single_value(out, params, "TLS ciphersuite", args->user[i]->tls_ciphersuite, 1);
+		print_single_value(out, params, "DTLS cipher", args->user[i]->dtls_ciphersuite, 1);
+		print_pair_value(out, params, "CSTP compression", args->user[i]->cstp_compr, "DTLS compression", args->user[i]->dtls_compr, 1);
 
+		print_separator(out, params);
 		/* user network info */
-		fputs("\n", out);
-		if (print_list_entries(out, "\tDNS:", args->user[i]->dns, args->user[i]->n_dns) < 0)
+		if (print_list_entries(out, params, "DNS", args->user[i]->dns, args->user[i]->n_dns, 1) < 0)
 			goto error_parse;
 
-		if (print_list_entries(out, "\tNBNS:", args->user[i]->nbns, args->user[i]->n_nbns) < 0)
+		if (print_list_entries(out, params, "NBNS", args->user[i]->nbns, args->user[i]->n_nbns, 1) < 0)
 			goto error_parse;
 
-		if ((r = print_list_entries(out, "\tRoutes:", args->user[i]->routes, args->user[i]->n_routes)) < 0)
+		if ((r = print_list_entries(out, params, "Routes", args->user[i]->routes, args->user[i]->n_routes, 1)) < 0)
 			goto error_parse;
 		if (r == 0) {
-			fprintf(out, "Routes: defaultroute\n");
+			print_single_value(out, params, "Routes", "defaultroute", 1);
 		}
 
-		if ((r = print_list_entries(out, "\tNo-routes:", args->user[i]->no_routes, args->user[i]->n_no_routes)) < 0)
+		if ((r = print_list_entries(out, params, "No-routes", args->user[i]->no_routes, args->user[i]->n_no_routes, 1)) < 0)
 			goto error_parse;
 
-		if (print_list_entries(out, "\tiRoutes:", args->user[i]->iroutes, args->user[i]->n_iroutes) < 0)
+		if (print_list_entries(out, params, "iRoutes", args->user[i]->iroutes, args->user[i]->n_iroutes, 0) < 0)
 			goto error_parse;
 
+		print_end_block(out, params, i<(args->n_user-1)?1:0);
+
 		at_least_one = 1;
 	}
 
+	if (HAVE_JSON(params))
+		fprintf(out, "]\n");
+
 	ret = 0;
 	goto cleanup;
 
@@ -838,15 +863,17 @@ int common_info_cmd(UserListRep * args)
 	goto cleanup;
  cleanup:
 	if (at_least_one == 0) {
-		fprintf(out, "user or ID not found\n");
+		if (NO_JSON(params))
+			fprintf(out, "user or ID not found\n");
 		ret = 2;
 	}
-	pager_stop(out);
+	if (init_pager)
+		pager_stop(out);
 
 	return ret;
 }
 
-int handle_show_user_cmd(struct unix_ctx *ctx, const char *arg)
+int handle_show_user_cmd(struct unix_ctx *ctx, const char *arg, cmd_params_st *params)
 {
 	int ret;
 	struct cmd_reply_st raw;
@@ -874,7 +901,7 @@ int handle_show_user_cmd(struct unix_ctx *ctx, const char *arg)
 	if (rep == NULL)
 		goto error;
 
-	ret = common_info_cmd(rep);
+	ret = common_info_cmd(rep, NULL, params);
 	if (ret < 0)
 		goto error;
 
@@ -891,7 +918,7 @@ int handle_show_user_cmd(struct unix_ctx *ctx, const char *arg)
 	return ret;
 }
 
-int handle_show_id_cmd(struct unix_ctx *ctx, const char *arg)
+int handle_show_id_cmd(struct unix_ctx *ctx, const char *arg, cmd_params_st *params)
 {
 	int ret;
 	struct cmd_reply_st raw;
@@ -923,7 +950,7 @@ int handle_show_id_cmd(struct unix_ctx *ctx, const char *arg)
 	if (rep == NULL)
 		goto error;
 
-	ret = common_info_cmd(rep);
+	ret = common_info_cmd(rep, NULL, params);
 	if (ret < 0)
 		goto error;
 
