@@ -183,21 +183,21 @@ typedef enum {
 struct group_cfg_st {
 	/* routes to be forwarded to the client */
 	char **routes;
-	unsigned int routes_size;
+	size_t routes_size;
 
 	/* routes that are excluded */
 	char **no_routes;
-	unsigned int no_routes_size;
+	size_t no_routes_size;
 
 	/* routes to be applied to the server */
 	char **iroutes;
-	unsigned int iroutes_size;
+	size_t iroutes_size;
 
 	char **dns;
-	unsigned int dns_size;
+	size_t dns_size;
 
 	char **nbns;
-	unsigned int nbns_size;
+	size_t nbns_size;
 
 	char *ipv4_network;
 	char *ipv6_network;
@@ -238,17 +238,17 @@ struct vpn_st {
 	unsigned int mtu;
 
 	char **routes;
-	unsigned int routes_size;
+	size_t routes_size;
 
 	/* excluded routes */
 	char **no_routes;
-	unsigned int no_routes_size;
+	size_t no_routes_size;
 
 	char **dns;
-	unsigned int dns_size;
+	size_t dns_size;
 
 	char **nbns;
-	unsigned int nbns_size;
+	size_t nbns_size;
 };
 
 #define MAX_AUTH_METHODS 4
@@ -291,16 +291,6 @@ struct cfg_st {
 	kkdcp_st *kkdcp;
 	unsigned int kkdcp_size;
 
-	char *pin_file;
-	char *srk_pin_file;
-	char **cert;
-	unsigned cert_size;
-	char **key;
-	unsigned key_size;
-
-	char *ca;
-	char *crl;
-	char *dh_params_file;
 	char *cert_user_oid;	/* The OID that will be used to extract the username */
 	char *cert_group_oid;	/* The OID that will be used to extract the groupname */
 
@@ -322,10 +312,10 @@ struct cfg_st {
 	char *default_select_group;
 
 	char **custom_header;
-	unsigned custom_header_size;;
+	size_t custom_header_size;;
 
 	char **split_dns;
-	unsigned split_dns_size;;
+	size_t split_dns_size;;
 
 
 	unsigned deny_roaming; /* whether a cookie is restricted to a single IP */
@@ -371,6 +361,8 @@ struct cfg_st {
 	size_t tx_per_sec;
 	unsigned net_priority;
 
+	char *crl;
+
 	unsigned output_buffer;
 	unsigned default_mtu;
 	unsigned predictable_ips; /* boolean */
@@ -387,7 +379,6 @@ struct cfg_st {
 #ifdef ANYCONNECT_CLIENT_COMPAT
 	char *xml_config_file;
 	char *xml_config_hash;
-	char *cert_hash;
 #endif
 
 	/* additional configuration files */
@@ -419,6 +410,22 @@ struct perm_cfg_st {
 	uid_t uid;
 	gid_t gid;
 
+	char *key_pin;
+	char *srk_pin;
+
+	char *pin_file;
+	char *srk_pin_file;
+	char **cert;
+	size_t cert_size;
+	char **key;
+	size_t key_size;
+#ifdef ANYCONNECT_CLIENT_COMPAT
+	char *cert_hash;
+#endif
+
+	char *ca;
+	char *dh_params_file;
+
 	char *listen_host;
 	char* unix_conn_file;
 	unsigned int port;
@@ -438,7 +445,7 @@ struct main_server_st;
 #define MAX_GROUPNAME_SIZE MAX_USERNAME_SIZE
 #define MAX_SESSION_DATA_SIZE (4*1024)
 
-#define MAX_CONFIG_ENTRIES 96
+#define DEFAULT_CONFIG_ENTRIES 96
 
 #include <tun.h>
 
