@@ -281,7 +281,7 @@ int get_auth_handler2(worker_st * ws, unsigned http_ver, const char *pmsg, unsig
 
 		/* send groups */
 		if (ws->config->group_list_size > 0 || ws->cert_groups_size > 0) {
-			ret = str_append_str(&str, "<select name=\"group_list\" label=\"GROUP:\">\n");
+			ret = str_append_str(&str, "<select name=\"group_list\" label=\"Group:\">\n");
 			if (ret < 0) {
 				ret = -1;
 				goto cleanup;
@@ -1006,14 +1006,14 @@ int post_common_handler(worker_st * ws, unsigned http_ver, const char *imsg)
 		ret =
 		    cstp_printf(ws,
 			       "Set-Cookie: webvpnc=bu:/&p:t&iu:1/&sh:%s&lu:/+CSCOT+/translation-table?textdomain%%3DAnyConnect%%26type%%3Dmanifest&fu:profiles%%2F%s&fh:%s; path=/; Secure\r\n",
-			       ws->config->cert_hash,
+			       ws->perm_config->cert_hash,
 			       ws->config->xml_config_file,
 			       ws->config->xml_config_hash);
 	} else {
 		ret =
 		    cstp_printf(ws,
 			       "Set-Cookie: webvpnc=bu:/&p:t&iu:1/&sh:%s; path=/; Secure\r\n",
-			       ws->config->cert_hash);
+			       ws->perm_config->cert_hash);
 	}
 
 	if (ret < 0)
@@ -1419,7 +1419,13 @@ int post_auth_handler(worker_st * ws, unsigned http_ver)
 
 		ireq.hostname = req->hostname;
 		ireq.ip = ws->remote_ip_str;
-		ireq.our_ip = get_our_ip(ws->conn_fd, our_ip_str);
+		if (ws->conn_type != SOCK_TYPE_UNIX) {
+			ireq.our_ip = get_our_ip(ws->conn_fd, our_ip_str);
+		} else if (ws->udp_state == UP_ACTIVE) {
+			/* if we use a UNIX socket then try to get our IP
+			 * from the UDP socket. */
+			ireq.our_ip = get_our_ip(ws->dtls_tptr.fd, our_ip_str);
+		}
 
 		sd = connect_to_secmod(ws);
 		if (sd == -1) {
@@ -1450,6 +1456,8 @@ int post_auth_handler(worker_st * ws, unsigned http_ver)
 			if (req->authorization == NULL || req->authorization_size <= 10) {
 				if (req->authorization != NULL)
 					oclog(ws, LOG_HTTP_DEBUG, "Invalid authorization data: %.*s", req->authorization_size, req->authorization);
+				else
+					oclog(ws, LOG_HTTP_DEBUG, "No authorization data");
 				goto auth_fail;
 			}
 			areq.password = req->authorization + 10;
@@ -1497,8 +1505,10 @@ int post_auth_handler(worker_st * ws, unsigned http_ver)
 			}
 
 			ws->auth_state = S_AUTH_REQ;
-		} else
+		} else {
+			oclog(ws, LOG_ERR, "No password provided");
 			goto auth_fail;
+		}
 	} else {
 		oclog(ws, LOG_ERR, "unexpected POST request in auth state %u",
 		      (unsigned)ws->auth_state);
