@@ -46,6 +46,11 @@
 #include <c-ctype.h>
 #include <worker-bandwidth.h>
 
+#ifdef __linux__
+/* for IPV6_PATHMTU */
+# include <linux/in6.h>
+#endif
+
 #include <vpn.h>
 #include "ipc.pb-c.h"
 #include <cookies.h>
@@ -657,15 +662,21 @@ int mtu_not_ok(worker_st * ws)
 		      ws->last_bad_mtu, ws->conn_mtu);
 	} else if (ws->proto == AF_INET6) { /* IPv6 */
 		int mtu;
-		socklen_t len = sizeof(mtu);
+#ifdef IPV6_PATHMTU
+		struct ip6_mtuinfo mtuinfo;
+		socklen_t len = sizeof(mtuinfo);
 
-		if (getsockopt(ws->dtls_tptr.fd, IPPROTO_IPV6, IPV6_MTU, &mtu, &len) < 0 || mtu < 1280) {
+		if (getsockopt(ws->dtls_tptr.fd, IPPROTO_IPV6, IPV6_PATHMTU, &mtuinfo, &len) < 0 || mtuinfo.ip6m_mtu < 1280) {
 			oclog(ws, LOG_INFO, "cannot obtain IPv6 MTU (was %u); disabling DTLS",
 			      ws->conn_mtu);
 			dtls_close(ws);
 			ws->udp_state = UP_DISABLED;
 			return -1;
 		}
+		mtu = mtuinfo.ip6m_mtu;
+#else
+		mtu = 1280; /* minimum IPv6 MTU */
+#endif
 
 		mtu -= CSTP_DTLS_OVERHEAD - ws->proto_overhead;
 
@@ -921,6 +932,8 @@ static int dtls_mainloop(worker_st * ws, struct timespec *tnow)
 
 			ws->last_dtls_rehandshake = tnow->tv_sec;
 		} else if (ret >= 1) {
+			/* where we receive any DTLS UDP packet we reset the state
+			 * to active */
 			ws->udp_state = UP_ACTIVE;
 
 			if (bandwidth_update
@@ -2087,6 +2100,12 @@ static int parse_cstp_data(struct worker_st *ws,
 		return -1;
 	}
 
+	if (buf[6] == AC_PKT_DATA && ws->udp_state == UP_ACTIVE) {
+		/* if we received a data packet in the CSTP channel we assume that
+		 * our peer wants to switch to it as the communication channel */
+		ws->udp_state = UP_INACTIVE;
+	}
+
 	ret = parse_data(ws, buf, buf_size, now, 0);
 	/* whatever we received treat it as DPD response.
 	 * it indicates that the channel is alive */
